#include "Terrain/ClipmapSampling.hlsl"

cbuffer UpsampleParameters
{
  float2 sourceOrigin;
  float2 updateSize;
  float2 destinationOffset;
  float noData;
  bool enableNoData;
};

struct VertexIn
{
  float4 Position : SV_Position;
  float2 TextureCoordinates : TEXCOORD0;
};

Texture2D coarseImagery;
SamplerState coarseImagerySampler;

float4 main(VertexIn In) : SV_Target
{
  if (enableNoData) {
    // We assume noData is only enabled for one-channel texture maps. When noData handling is enabled, samples
    // containing any noData texels will result in only noData being written to the output texture, instead
    // of interpolating between noData and neighboring values. The noData value can then be properly detected
    // in the final output shaders.
    float s = sampleLinearNoData(coarseImagery, coarseImagerySampler, In.TextureCoordinates, noData);

    return s.rrrr;
  } else {
    return coarseImagery.Sample(coarseImagerySampler, In.TextureCoordinates);
  }
}
