// Computes a blended color from the current and the coarser levels imagery maps.
float4 computeColor(int index, in VertexIn In)
{
  float4 texCoords = In.ImageryTexCoords[index];
#ifdef GLSLANG
    texCoords.yw = (float2)1.0 - texCoords.yw;
#endif
  float4 fineColor = imageryTexture[index].Sample(imagerySampler, texCoords.xy);
  float4 coarseColor = coarseImageryTexture[index].Sample(imagerySampler, texCoords.zw);

  return lerp(fineColor, coarseColor, In.BlendFactor);
}

float4 computeDiffuseColor(in VertexIn In)
{
  float4 diffuseColor = float4(0, 0, 0, 0);

  if (showImagery) {
    [unroll]
    for (int i = 0; i < NUM_ACTIVE_TEXTURES; i++) {
      diffuseColor += computeColor(i, In);
    }

    diffuseColor = diffuseColor / float(NUM_ACTIVE_TEXTURES);
  } else {
    diffuseColor = color;
  }

  diffuseColor = showLevelColors ? lerp(diffuseColor, color, 0.5) : diffuseColor;

  diffuseColor = showBlendRegions ? lerp(diffuseColor, blendRegionColor, In.BlendFactor) : diffuseColor;

  return float4(diffuseColor.rgb, saturate(diffuseColor.a));
}
